@@ -12,6 +12,6 @@ struct cardart *cardart_load_cardart3 (char *cardname, char *expansions, int);
 void cardart_destroy (struct cardart *);
 
 GdkPixbuf *cardart_get_card_image (struct cardinfo *);
-GdkPixbuf *cardart_get_unscaled_art (struct cardart *);
-GdkPixbuf *cardart_get_small_art (struct cardart *, int width, int height);
+GdkPixbuf *cardart_get_unscaled_art (struct cardinfo *);
+GdkPixbuf *cardart_get_small_art (struct cardinfo *, int width, int height);
 struct cardart *cardart_get_null_art (void);
