@@ -9,6 +9,7 @@
 #define COL_LAND_BIT            0x80
 
 struct cardbase {
+    char *filename;
     int num_cards;
     struct cardinfo *cards;
     int num_extras;
@@ -33,3 +34,4 @@ struct cardbase *cardbase_new_from_ort (const char *filename);
 int cardbase_find_globalid (struct cardbase *db, const char *name);
 struct cardinfo *cardbase_get_card (struct cardbase *db, int globalid);
 int cardbase_add_card (struct cardbase *db, const char *, char *, int);
+void cardbase_destroy (struct cardbase *db);
