@@ -24,4 +24,4 @@ void deck_set_token (struct deck *deck, int idx, int qty, int globalid);
 int deck_lookup (struct deck *deck, int cid);
 void deck_remove_tokens (struct deck *deck);
 void deck_destroy (struct deck *);
-void deck_write_file (char *filename, struct deck *, struct cardbase *);
+void deck_write_file (const char *filename, struct deck *, struct cardbase *);
