@@ -1,5 +1,6 @@
 /*
  *  Copyright (c) 1999  Salvatore Valente <svalente@mit.edu>
+ *  some portions copyright (c) 2005 Federico Poloni <f.poloni@sns.it>
  *
  *  This program is free software.  You can modify and distribute it under
  *  the terms of the GNU General Public License.  There is no warranty.
@@ -24,6 +25,8 @@
 #include "playarea.h"
 #include "cardart.h"
 #include "prefs.h"
+#include "expansions.h"
+#include "viewcard.h"
 
 #if GTK_MAJOR_VERSION >= 2
 #define KEY_EVENT "key_release_event"
@@ -79,6 +82,7 @@ void do_input_dialog (struct game *game, const char *title,
     struct input_args *args;
 
     dialog = gtk_dialog_new ();
+    game_make_transient (game, dialog);
     dp = GTK_DIALOG (dialog);
     gtk_container_set_border_width (GTK_CONTAINER (dp->vbox), 10);
     gtk_window_set_title (GTK_WINDOW (dialog), title);
@@ -153,6 +157,7 @@ void do_verify_dialog (struct game *game, const char *title,
 
     dialog = gtk_dialog_new ();
     gtk_window_set_title (GTK_WINDOW (dialog), title);
+    game_make_transient (game, dialog);
     dp = GTK_DIALOG (dialog);
     gtk_container_set_border_width (GTK_CONTAINER (dialog), 15);
     gtk_box_set_spacing (GTK_BOX (dp->vbox), 10);
@@ -221,7 +226,7 @@ void do_life_dialog (struct game *game, int what_todo)
 	break;
     }
     do_input_dialog (game, title, prompt, NULL, life_dialog_callback,
-		     (gpointer) what_todo);
+		     GINT_TO_POINTER(what_todo));
 }
 
 static int
@@ -229,7 +234,7 @@ life_dialog_callback (struct game *game, gpointer data, const char *text)
 {
     int what_todo, old_life, new_life;
 
-    what_todo = (int) data;
+    what_todo = GPOINTER_TO_INT(data);
     old_life = game->player[game->local_player]->life;
     if (*text == '+') {
 	what_todo = GAIN_LIFE;
@@ -267,6 +272,7 @@ void do_library_dialog (struct game *game)
 
     dialog = gtk_dialog_new ();
     gtk_window_set_title (GTK_WINDOW (dialog), "Library Options");
+    game_make_transient (game, dialog);
     dp = GTK_DIALOG (dialog);
     gtk_container_set_border_width (GTK_CONTAINER (dialog), 15);
     gtk_box_set_spacing (GTK_BOX (dp->vbox), 10);
@@ -322,6 +328,7 @@ struct zone_dialog_args {
     struct game *game;
     int pid;
     int znum;
+    int private;
 };
 
 static gint zone_dialog_delete (GtkWidget *w, GdkEvent *event,
@@ -333,6 +340,14 @@ static void do_zone_move_card (GtkWidget *, struct zone_dialog_args *);
 static void do_move_all_to_bottom (GtkWidget *, struct zone_dialog_args *);
 
 void do_zone_dialog (struct game *game, int pid, int znum, int quantity)
+{
+    struct zone *zone = game->player[pid]->zone[znum];
+    do_qzone_dialog (game, pid, znum, quantity,
+		     ((zone->flags & ZONE_PRIVATE) != 0));
+}
+
+void do_qzone_dialog (struct game *game, int pid, int znum, int quantity,
+		      int private)
 {
     struct player *me;
     struct zone *zone;
@@ -347,7 +362,7 @@ void do_zone_dialog (struct game *game, int pid, int znum, int quantity)
     if (zone == NULL)
 	return;
     if (zone->dialog != NULL) {
-	gdk_window_raise (zone->dialog->window);
+	gtk_window_present (GTK_WINDOW (zone->dialog));
 	return;
     }
     if (pid == game->local_player) {
@@ -362,6 +377,7 @@ void do_zone_dialog (struct game *game, int pid, int znum, int quantity)
     if (quantity > 0)
 	sprintf (title+strlen (title), " (%d)", quantity);
     dialog = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+    game_make_transient (game, dialog);
 
     args = malloc (sizeof (struct zone_dialog_args));
     args->dialog = dialog;
@@ -369,6 +385,7 @@ void do_zone_dialog (struct game *game, int pid, int znum, int quantity)
     args->game = game;
     args->pid = pid;
     args->znum = znum;
+    args->private = private;
 
     gtk_signal_connect (GTK_OBJECT (dialog), "delete_event",
 			GTK_SIGNAL_FUNC (zone_dialog_delete), args);
@@ -430,7 +447,7 @@ void do_zone_dialog (struct game *game, int pid, int znum, int quantity)
     zone->dialog = dialog;
     gtk_widget_show_all (dialog);
 
-    if ((zone->flags & ZONE_PRIVATE) != 0) {
+    if (args->private) {
 	game_inform_peeking (game, game->local_player, pid, znum, quantity, 1);
 	send_inform_peeking (game, pid, znum, quantity, 1);
     }
@@ -478,7 +495,7 @@ static void close_zone_dialog (struct zone_dialog_args *args, int do_close)
     int qty, do_shuffle;
 
     zone = game->player[args->pid]->zone[args->znum];
-    if ((zone->flags & ZONE_PRIVATE) != 0) {
+    if (args->private) {
 	qty = zone->top_cards_requested;
 	game_inform_peeking (game, game->local_player, args->pid,
 			     args->znum, qty, 0);
@@ -617,6 +634,7 @@ void do_create_card_dialog (struct game *game)
     dp = GTK_DIALOG (args->dialog);
     gtk_container_set_border_width (GTK_CONTAINER (dp->vbox), 10);
     gtk_window_set_title (GTK_WINDOW (args->dialog), "Create Card");
+    game_make_transient (game, args->dialog);
 
     table = gtk_table_new (2, 3, FALSE);
 
@@ -708,118 +726,43 @@ static void create_card_cancel (GtkWidget *dialog, struct create_args *args)
 
 /* ----------------------------------------------------------------- */
 
-static GtkWidget *pixbuf_to_widget (GdkPixbuf *, GtkWidget *);
-static void do_card_image_dialog (GtkWidget *w, struct cardinfo *);
+static void clear_view_card_dialog (GtkWidget *w, struct game *game);
 
-void do_view_card_dialog (struct game *game, int cid)
+/**
+ * Display the given card in the game's View Card dialog.
+ * Be sure to note when the window is created and destroyed.
+ */
+void
+do_view_card_dialog (struct game *game, int cid)
 {
-    struct cardinfo *info;
-    GtkWidget *dialog, *picwidget, *name_label, *cost_label, *text_label;
-    GtkWidget *cardtype_label, *flavor_label, *pow_label;
-    GtkWidget *layout, *namebox, *frame;
-    GtkWidget *botbox, *close_button, *comp_button;
-    GdkPixbuf *pixbuf;
-
-    dialog = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-    gtk_window_set_title (GTK_WINDOW (dialog), "View Card");
-
-    info = game_get_cardinfo (game, cid);
-    if (info->art == NULL)
-	info->art = cardart_load_cardart (info->name, info->expansion);
-
-    /*  Do the old View Card dialog.  */
-    gtk_container_set_border_width (GTK_CONTAINER (dialog), 10);
-
-    pixbuf = cardart_get_unscaled_art (info->art);
-    picwidget = NULL;
-    if (pixbuf != NULL)
-	picwidget = pixbuf_to_widget (pixbuf, game->playarea->darea);
-
-    name_label = gtk_label_new (info->name);
-    cost_label = gtk_label_new (info->cost);
-    cardtype_label = gtk_label_new (info->cardtype);
-    text_label = gtk_label_new (info->text);
-    flavor_label = gtk_label_new (info->flavor);
-    pow_label = gtk_label_new (info->pow_tgh);
-
-    gtk_misc_set_alignment (GTK_MISC (name_label), 0, 0);
-    gtk_misc_set_alignment (GTK_MISC (cost_label), 1, 0);
-    gtk_misc_set_alignment (GTK_MISC (cardtype_label), 0, 0);
-    gtk_misc_set_alignment (GTK_MISC (text_label), 0, 0);
-    gtk_misc_set_alignment (GTK_MISC (flavor_label), 0, 0);
-    gtk_misc_set_alignment (GTK_MISC (pow_label), 1, 0);
-
-    gtk_label_set_line_wrap (GTK_LABEL (text_label), TRUE);
-    gtk_label_set_line_wrap (GTK_LABEL (flavor_label), TRUE);
-
-    close_button = new_wide_button ("Close", 6);
-    gtk_signal_connect_object (GTK_OBJECT (close_button), "clicked",
-			       GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			       GTK_OBJECT (dialog));
-    comp_button = new_wide_button ("Card Image", 6);
-    gtk_signal_connect (GTK_OBJECT (comp_button), "clicked",
-			GTK_SIGNAL_FUNC (do_card_image_dialog), info);
-
-    layout = gtk_vbox_new (FALSE, 5);
-    gtk_box_set_spacing (GTK_BOX (layout), 5);
-    namebox = gtk_hbox_new (FALSE, 0);
-    gtk_box_pack_start (GTK_BOX (namebox), name_label, FALSE, FALSE, 0);
-    gtk_box_pack_start (GTK_BOX (namebox), cost_label, TRUE, TRUE, 0);
-    gtk_box_pack_start (GTK_BOX (layout), namebox, FALSE, FALSE, 0);
-    if (picwidget != NULL)
-	gtk_box_pack_start (GTK_BOX (layout), picwidget, FALSE, FALSE, 0);
-    gtk_box_pack_start (GTK_BOX (layout), cardtype_label, TRUE, FALSE, 0);
-    frame = gtk_frame_new (NULL);
-    gtk_container_add (GTK_CONTAINER (frame), text_label);
-    gtk_misc_set_padding (GTK_MISC (text_label), 5, 5);
-    gtk_box_pack_start (GTK_BOX (layout), frame, FALSE, FALSE, 0);
-    frame = gtk_frame_new (NULL);
-    gtk_container_add (GTK_CONTAINER (frame), flavor_label);
-    gtk_misc_set_padding (GTK_MISC (flavor_label), 5, 5);
-    gtk_box_pack_start (GTK_BOX (layout), frame, FALSE, FALSE, 0);
-    botbox = gtk_hbox_new (FALSE, 10);
-    gtk_box_pack_start (GTK_BOX (botbox), close_button, FALSE, FALSE, 0);
-    if (comp_button != NULL)
-	gtk_box_pack_start (GTK_BOX (botbox), comp_button, FALSE, FALSE, 0);
-    gtk_box_pack_start (GTK_BOX (botbox), pow_label, TRUE, TRUE, 0);
-    gtk_box_pack_start (GTK_BOX (layout), botbox, FALSE, FALSE, 0);
-    gtk_container_add (GTK_CONTAINER (dialog), layout);
-    gtk_widget_show_all (dialog);
+    do_view_card_dialog_from_cardinfo (game, game_get_cardinfo (game, cid));
 }
 
-static GtkWidget *pixbuf_to_widget (GdkPixbuf *pixbuf, GtkWidget *widget)
+void
+do_view_card_dialog_from_cardinfo (struct game *game, struct cardinfo *info)
 {
-    GdkPixmap *pixmap;
-    int width, height;
-    GtkWidget *picwidget;
+    GtkWidget *dialog;
 
-    width = gdk_pixbuf_get_width (pixbuf);
-    height = gdk_pixbuf_get_height (pixbuf);
-    pixmap = gdk_pixmap_new (widget->window, width, height, -1);
-    gdk_pixbuf_render_to_drawable (pixbuf, pixmap, widget->style->white_gc,
-				   0, 0, 0, 0, width, height,
-				   GDK_RGB_DITHER_NORMAL, 0, 0);
-    picwidget = gtk_pixmap_new (pixmap, NULL);
-    gdk_pixmap_unref (pixmap);
-    return (picwidget);
+    if (get_int_preference (game->prefs, "view_card_windows", 0) != 0) {
+	dialog = viewcard_new_window ();
+    } else {
+	if (game->view_card_dialog == NULL) {
+	    game->view_card_dialog = viewcard_new_window ();
+	    gtk_signal_connect (GTK_OBJECT (game->view_card_dialog),
+				"destroy",
+				GTK_SIGNAL_FUNC (clear_view_card_dialog),
+				game);
+	}
+	dialog = game->view_card_dialog;
+    }
+    game_make_transient (game, dialog);
+    viewcard_display_cardinfo (dialog, info);
 }
 
-static void do_card_image_dialog (GtkWidget *w, struct cardinfo *info)
+static void
+clear_view_card_dialog (GtkWidget *w, struct game *game)
 {
-    GdkPixbuf *pixbuf;
-    GtkWidget *dialog, *picwidget;
-
-    if (info->art == NULL)
-	info->art = cardart_load_cardart3 (info->name, info->expansion, TRUE);
-    pixbuf = cardart_get_card_image (info);
-    if (pixbuf == NULL)
-	return;
-
-    dialog = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-    gtk_window_set_title (GTK_WINDOW (dialog), "View Card Image");
-    picwidget = pixbuf_to_widget (pixbuf, w);
-    gtk_container_add (GTK_CONTAINER (dialog), picwidget);
-    gtk_widget_show_all (dialog);
+    game->view_card_dialog = NULL;
 }
 
 /* ----------------------------------------------------------------- */
@@ -862,6 +805,7 @@ void do_preferences_dialog (struct game *game)
 
     args->dialog = gtk_dialog_new ();
     gtk_window_set_title (GTK_WINDOW (args->dialog), "Preferences");
+    game_make_transient (game, args->dialog);
     dp = GTK_DIALOG (args->dialog);
     gtk_container_set_border_width (GTK_CONTAINER (args->dialog), 10);
     gtk_box_set_spacing (GTK_BOX (dp->vbox), 10);
@@ -1070,7 +1014,7 @@ static void image_browse_ok (GtkWidget *dialog, gpointer data)
     gtk_entry_set_text (GTK_ENTRY (args->image_entry), filename);
     gtk_widget_destroy (dialog);
 }
-    
+
 /* ----------------------------------------------------------------- */
 
 struct sblist_line {
@@ -1079,9 +1023,13 @@ struct sblist_line {
 };
 
 struct sb_list {
+    struct game *game;
     GtkWidget *scrollbox;
     GtkWidget *clist;
+    GtkWidget *button;
+    GtkWidget *card_count;
     int num_lines;
+    int num_cards;
     struct sblist_line *lineinfo;
 };
 
@@ -1094,18 +1042,29 @@ struct sideboard_args {
 };
 
 static void create_sb_list (struct sb_list *, int, int *, struct game *);
+static void sideboard_select_row (GtkCList *, gint, gint, GdkEventButton *,
+				  struct sb_list *);
 static void sideboard_trade (GtkWidget *, struct sideboard_args *);
+static void sideboard_add_to_deck (GtkWidget *, struct sideboard_args *);
+static void sideboard_remove_from_deck (GtkWidget *, struct sideboard_args *);
+static void sideboard_save_deck (GtkWidget *, struct sideboard_args *);
 static void decrement_line (struct sb_list *listp, int idx);
 static void increment_globalid (struct sb_list *, int, struct cardbase *);
 static void sideboard_dialog_ok (GtkWidget *, struct sideboard_args *);
-static void sblist_to_cardlist (struct sb_list *listp, int *cardlist);
+struct deck *sideboard_args_to_deck (struct sideboard_args *args);
 static void sideboard_dialog_cancel (GtkWidget *, struct sideboard_args *);
 
-void do_sideboard_dialog (struct game *game)
+/**
+ * Creating is true if we are creating the deck (that is, we can freely
+ * add/remove cards from it).
+ */
+void
+do_sideboard_dialog (struct game *game, int creating)
 {
     struct deck *deck;
     struct sideboard_args *args;
-    GtkWidget *hbox, *vbox, *button;
+    GtkWidget *hbox, *vbox, *button, *add_button, *remove_button, *card_count;
+    char *title;
 
     deck = game->player[game->local_player]->deck;
     if (deck == NULL)
@@ -1113,7 +1072,12 @@ void do_sideboard_dialog (struct game *game)
     args = malloc (sizeof (struct sideboard_args));
     args->game = game;
     args->dialog = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-    gtk_window_set_title (GTK_WINDOW (args->dialog), "Sideboard");
+    gtk_window_set_transient_for (GTK_WINDOW (args->dialog),
+				  GTK_WINDOW (game->new_game_dialog));
+
+    title = (creating ? "Create Deck" : "Sideboard");
+    gtk_window_set_title (GTK_WINDOW (args->dialog), title);
+
     gtk_container_set_border_width (GTK_CONTAINER (args->dialog), 10);
     hbox = gtk_hbox_new (FALSE, 10);
     create_sb_list (&args->deck_list, deck->size, deck->cardlist, game);
@@ -1124,6 +1088,26 @@ void do_sideboard_dialog (struct game *game)
     gtk_signal_connect (GTK_OBJECT (button), "clicked",
 			GTK_SIGNAL_FUNC (sideboard_trade), args);
     gtk_box_pack_start (GTK_BOX (vbox), button, TRUE, FALSE, 0);
+    card_count = add_button = remove_button = NULL;
+    if (creating)
+    {
+	card_count = gtk_label_new ("Deck:\n0 cards");
+	gtk_box_pack_start (GTK_BOX (vbox), card_count, TRUE, FALSE, 0);
+	add_button = new_wide_button ("<<", 6);
+	gtk_signal_connect (GTK_OBJECT (add_button), "clicked",
+			    GTK_SIGNAL_FUNC (sideboard_add_to_deck),
+			    args);
+	gtk_box_pack_start (GTK_BOX (vbox), add_button, TRUE, FALSE, 0);
+	remove_button = new_wide_button (">>", 6);
+	gtk_signal_connect (GTK_OBJECT (remove_button), "clicked",
+			    GTK_SIGNAL_FUNC (sideboard_remove_from_deck),
+			    args);
+	gtk_box_pack_start (GTK_BOX (vbox), remove_button, TRUE, FALSE, 0);
+	button = new_wide_button ("Save deck", 6);
+	gtk_signal_connect (GTK_OBJECT (button), "clicked",
+			    GTK_SIGNAL_FUNC (sideboard_save_deck), args);
+	gtk_box_pack_start (GTK_BOX (vbox), button, TRUE, FALSE, 0);
+    }
     button = new_wide_button ("Ok", 6);
     gtk_signal_connect (GTK_OBJECT (button), "clicked",
 			GTK_SIGNAL_FUNC (sideboard_dialog_ok), args);
@@ -1138,20 +1122,33 @@ void do_sideboard_dialog (struct game *game)
 			FALSE, FALSE, 0);
     gtk_container_add (GTK_CONTAINER (args->dialog), hbox);
     args->changed = FALSE;
+    args->deck_list.card_count = card_count;
+    args->deck_list.button = remove_button;
+    args->sb_list.button = add_button;
     gtk_widget_show_all (args->dialog);
-    display_message (game, "+ %s is sideboarding.",
-		     game->player[game->local_player]->name);
-    send_inform_sideboard (game);
+    if (creating)
+    {
+	display_message (game, "* %s drafted a Sealed Deck game.",
+			 game->player[game->local_player]->name);
+	send_drafted_sealed (game);
+    }
+    else
+    {
+	display_message (game, "+ %s is sideboarding.",
+			 game->player[game->local_player]->name);
+	send_inform_sideboard (game);
+    }
 }
 
 static void
 create_sb_list (struct sb_list *listp, int size, int *cardlist,
 		struct game *game)
 {
-    int i, globalid, count;
+    int globalid, count;
+    char buf[24], *sarr[2];
     struct cardinfo *info;
-    char buf[10], *sarr[2];
 
+    listp->game = game;
     listp->scrollbox = gtk_scrolled_window_new (NULL, NULL);
     gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (listp->scrollbox),
 				    GTK_POLICY_AUTOMATIC,
@@ -1164,15 +1161,18 @@ create_sb_list (struct sb_list *listp, int size, int *cardlist,
 				  GTK_SELECTION_SINGLE);
     gtk_clist_set_column_width (GTK_CLIST (listp->clist), 0, 15);
     gtk_clist_set_column_width (GTK_CLIST (listp->clist), 1, 0);
+    gtk_clist_set_button_actions (GTK_CLIST (listp->clist),
+				  2, GTK_BUTTON_SELECTS);
 
-    listp->lineinfo = malloc (sizeof (struct sblist_line) * size);
+    listp->button = NULL;
+    listp->card_count = NULL;
     listp->num_lines = 0;
-
-    i = 0;
-    while (i < size) {
-	globalid = cardlist[i];
+    listp->num_cards = 0;
+    listp->lineinfo = malloc (sizeof (struct sblist_line) * size);
+    while (listp->num_cards < size) {
+	globalid = cardlist[listp->num_cards];
 	count = 1;
-	while (cardlist[i+count] == globalid)
+	while (cardlist[listp->num_cards+count] == globalid)
 	    count++;
 	info = cardbase_get_card (game->cardbase, globalid);
 	sprintf (buf, "%d", count);
@@ -1182,11 +1182,44 @@ create_sb_list (struct sb_list *listp, int size, int *cardlist,
 	listp->lineinfo[listp->num_lines].qty = count;
 	listp->lineinfo[listp->num_lines].globalid = globalid;
 	listp->num_lines++;
-	i += count;
+	listp->num_cards += count;
+    }
+
+    gtk_signal_connect (GTK_OBJECT (listp->clist), "select-row",
+			GTK_SIGNAL_FUNC (sideboard_select_row), listp);
+    gtk_signal_connect (GTK_OBJECT (listp->clist), "unselect-row",
+			GTK_SIGNAL_FUNC (sideboard_select_row), listp);
+}
+
+static void
+sideboard_select_row (GtkCList *clist, gint row, gint column,
+		      GdkEventButton *event, struct sb_list *listp)
+{
+    struct game *game = listp->game;
+    int rnum, cnum, globalid;
+    struct cardinfo *info;
+
+    if ((event == NULL) ||
+	(!gtk_clist_get_selection_info (clist, event->x, event->y,
+					&rnum, &cnum)) ||
+	(rnum != row))
+	return;
+    if (event->type == GDK_2BUTTON_PRESS && event->button == 1)
+    {
+	if (listp->button != NULL)
+	    gtk_button_clicked (GTK_BUTTON (listp->button));
+	return;
+    }
+    if (event->type == GDK_BUTTON_RELEASE && event->button == 3)
+    {
+	globalid = listp->lineinfo[row].globalid;
+	info = cardbase_get_card (game->cardbase, globalid);
+	do_view_card_dialog_from_cardinfo (game, info);
     }
 }
 
-static void sideboard_trade (GtkWidget *w, struct sideboard_args *args)
+static void
+sideboard_trade (GtkWidget *w, struct sideboard_args *args)
 {
     int idx, sb_idx, left_gid, right_gid;
 
@@ -1207,12 +1240,46 @@ static void sideboard_trade (GtkWidget *w, struct sideboard_args *args)
     args->changed = TRUE;
 }
 
-static void decrement_line (struct sb_list *listp, int idx)
+static void
+sideboard_add_to_deck (GtkWidget *w, struct sideboard_args *args)
 {
-    char buf[10];
+    int sb_idx, right_gid;
+    sb_idx = gtk_clist_get_selected_row (GTK_CLIST (args->sb_list.clist));
+    if (sb_idx < 0)
+	return;
+    right_gid = args->sb_list.lineinfo[sb_idx].globalid;
+    decrement_line (&args->sb_list, sb_idx);
+    increment_globalid (&args->deck_list, right_gid,
+			args->game->cardbase);
+    args->changed = TRUE;
+}
+
+static void
+sideboard_remove_from_deck (GtkWidget *w, struct sideboard_args *args)
+{
+    int idx, left_gid;
+    idx = gtk_clist_get_selected_row (GTK_CLIST (args->deck_list.clist));
+    if (idx < 0)
+	return;
+    left_gid = args->deck_list.lineinfo[idx].globalid;
+    decrement_line (&args->deck_list, idx);
+    increment_globalid (&args->sb_list, left_gid,
+			args->game->cardbase);
+    args->changed = TRUE;
+}
+
+static void
+decrement_line (struct sb_list *listp, int idx)
+{
+    char buf[24];
     int i;
 
     listp->lineinfo[idx].qty--;
+    listp->num_cards--;
+    if (listp->card_count != NULL) {
+	sprintf (buf, "Deck:\n%d cards", listp->num_cards);
+	gtk_label_set_text (GTK_LABEL(listp->card_count), buf);
+    }
     if (listp->lineinfo[idx].qty > 0) {
 	sprintf (buf, "%d", listp->lineinfo[idx].qty);
 	gtk_clist_set_text (GTK_CLIST (listp->clist), idx, 0, buf);
@@ -1224,13 +1291,19 @@ static void decrement_line (struct sb_list *listp, int idx)
     gtk_clist_remove (GTK_CLIST (listp->clist), idx);
 }
 
-static void increment_globalid (struct sb_list *listp, int globalid,
-				struct cardbase *cardbase)
+static void
+increment_globalid (struct sb_list *listp, int globalid,
+		    struct cardbase *cardbase)
 {
     int idx;
-    char buf[10], *sarr[2];
+    char buf[24], *sarr[2];
     struct cardinfo *info;
 
+    listp->num_cards++;
+    if (listp->card_count != NULL) {
+	sprintf (buf, "Deck:\n%d cards", listp->num_cards);
+	gtk_label_set_text (GTK_LABEL(listp->card_count), buf);
+    }
     for (idx = 0; idx < listp->num_lines; idx++) {
 	if (listp->lineinfo[idx].globalid == globalid) {
 	    listp->lineinfo[idx].qty++;
@@ -1253,10 +1326,95 @@ static void increment_globalid (struct sb_list *listp, int globalid,
     gtk_clist_append (GTK_CLIST (listp->clist), sarr);
 }
 
-static void sideboard_dialog_ok (GtkWidget *w, struct sideboard_args *args)
+#if (GTK_MAJOR_VERSION>2) || (GTK_MAJOR_VERSION==2&&GTK_MINOR_VERSION>2)
+static void do_save_deck (GtkDialog *, gint, struct sideboard_args *);
+
+static void
+sideboard_save_deck (GtkWidget *w, struct sideboard_args *args)
+{
+    GtkWidget *dialog;
+    GtkFileFilter *filter;
+
+    dialog = gtk_file_chooser_dialog_new ("Save Deck",
+			GTK_WINDOW (args->dialog),
+			GTK_FILE_CHOOSER_ACTION_SAVE,
+			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+			GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
+			NULL);
+    gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
+    filter = gtk_file_filter_new ();
+    gtk_file_filter_add_pattern (filter, "*.dec");
+    gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (dialog), filter);
+    gtk_signal_connect (GTK_OBJECT (dialog), "response",
+			GTK_SIGNAL_FUNC (do_save_deck), args);
+    gtk_widget_show_all (dialog);
+}
+
+static void
+do_save_deck (GtkDialog *dialog, gint arg, struct sideboard_args *args)
+{
+    char *filename;
+    struct deck *deck;
+
+    switch (arg) {
+    case GTK_RESPONSE_ACCEPT:
+	filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
+	deck = sideboard_args_to_deck (args);
+	deck_write_file (filename, deck, args->game->cardbase);
+	deck_destroy (deck);
+	g_free (filename);
+	/* fallthrough */
+    case GTK_RESPONSE_CANCEL:
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+	break;
+    }
+}
+#else
+static void do_save_deck (GtkWidget *w, GtkWidget *dialog);
+
+static void
+sideboard_save_deck (GtkWidget *w, struct sideboard_args *args)
+{
+    GtkWidget *dialog;
+    GtkFileSelection *fsp;
+
+    dialog = gtk_file_selection_new ("Save Deck");
+    gtk_window_set_transient_for (GTK_WINDOW (dialog),
+				  GTK_WINDOW (args->dialog));
+    gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
+    gtk_object_set_user_data (GTK_OBJECT (dialog), args);
+    fsp = GTK_FILE_SELECTION (dialog);
+    gtk_signal_connect (GTK_OBJECT (fsp->ok_button), "clicked",
+			GTK_SIGNAL_FUNC (do_save_deck), dialog);
+    gtk_signal_connect_object (GTK_OBJECT (fsp->cancel_button), "clicked",
+			       GTK_SIGNAL_FUNC (gtk_widget_destroy),
+			       GTK_OBJECT (dialog));
+    gtk_widget_show_all (dialog);
+}
+
+static void
+do_save_deck (GtkWidget *w, GtkWidget *dialog)
+{
+    const gchar *filename;
+    struct sideboard_args *args;
+    struct deck *deck;
+
+    filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (dialog));
+    if (filename == NULL)
+	return;
+    args = gtk_object_get_user_data (GTK_OBJECT (dialog));
+    deck = sideboard_args_to_deck (args);
+    deck_write_file (filename, deck, args->game->cardbase);
+    deck_destroy (deck);
+    gtk_widget_destroy (dialog);
+}
+#endif
+
+static void
+sideboard_dialog_ok (GtkWidget *w, struct sideboard_args *args)
 {
     struct game *game;
-    struct deck *deck;
+    struct player *me;
 
     if (args->changed) {
 	game = args->game;
@@ -1265,23 +1423,38 @@ static void sideboard_dialog_ok (GtkWidget *w, struct sideboard_args *args)
 			       "during a game.");
 	    return;
 	}
-	deck = game->player[game->local_player]->deck;
-	sblist_to_cardlist (&args->deck_list, deck->cardlist);
-	sblist_to_cardlist (&args->sb_list, deck->sb_cardlist);
+	me = game->player[game->local_player];
+	if (me->deck != NULL)
+	    deck_destroy (me->deck);
+	me->deck = sideboard_args_to_deck (args);
     }
     sideboard_dialog_cancel (w, args);
 }
 
-static void sblist_to_cardlist (struct sb_list *listp, int *cardlist)
+struct deck *
+sideboard_args_to_deck (struct sideboard_args *args)
 {
-    int count, idx, x;
+    struct deck *deck;
+    int idx;
 
-    for (idx = count = 0; idx < listp->num_lines; idx++)
-	for (x = 0; x < listp->lineinfo[idx].qty; x++)
-	    cardlist[count++] = listp->lineinfo[idx].globalid;
+    deck = deck_new ();
+    for (idx = 0; idx < args->deck_list.num_lines; idx++) {
+	deck_set_card (deck, deck->size,
+		       args->deck_list.lineinfo[idx].qty,
+		       args->deck_list.lineinfo[idx].
+		       globalid);
+    }
+    for (idx = 0; idx < args->sb_list.num_lines; idx++) {
+	deck_set_sb_card (deck, deck->sb_size,
+			  args->sb_list.lineinfo[idx].qty,
+			  args->sb_list.lineinfo[idx].
+			  globalid);
+    }
+    return (deck);
 }
 
-static void sideboard_dialog_cancel (GtkWidget *w, struct sideboard_args *args)
+static void
+sideboard_dialog_cancel (GtkWidget *w, struct sideboard_args *args)
 {
     free (args->deck_list.lineinfo);
     free (args->sb_list.lineinfo);
@@ -1291,53 +1464,127 @@ static void sideboard_dialog_cancel (GtkWidget *w, struct sideboard_args *args)
 
 /* ----------------------------------------------------------------- */
 
-static void do_cardinfo_ok (GtkWidget *w, GtkWidget *dialog);
+static void finish_find_cardinfo (GtkWidget *, const char *, struct game *);
 static int is_ascii_file (const char *filename);
 
-void do_find_cardinfo_dialog (struct game *game)
+#if (GTK_MAJOR_VERSION>2) || (GTK_MAJOR_VERSION==2&&GTK_MINOR_VERSION>2)
+static void
+find_cardinfo_response (GtkDialog *dialog, gint arg, struct game *game)
 {
-    GtkWidget *filew;
-    GtkFileSelection *fsp;
-
-    filew = gtk_file_selection_new ("Load Magic Card Database");
-    gtk_object_set_user_data (GTK_OBJECT (filew), game);
-    fsp = GTK_FILE_SELECTION (filew);
-    gtk_signal_connect (GTK_OBJECT (fsp->ok_button), "clicked",
-			GTK_SIGNAL_FUNC (do_cardinfo_ok), filew);
-    gtk_signal_connect_object (GTK_OBJECT (fsp->cancel_button), "clicked",
-			       GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			       GTK_OBJECT (filew));
-    gtk_widget_show (filew);
+    const char *filename;
+    switch (arg) {
+    case GTK_RESPONSE_ACCEPT:
+	filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
+	finish_find_cardinfo (GTK_WIDGET (dialog), filename, game);
+	break;
+    case GTK_RESPONSE_CANCEL:
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+	break;
+    }
 }
 
-static void do_cardinfo_ok (GtkWidget *w, GtkWidget *dialog)
+void
+do_find_cardinfo_dialog (struct game *game)
 {
-    struct game *game;
-    const char *filename;
+    GtkWidget *dialog;
 
-    game = gtk_object_get_user_data (GTK_OBJECT (dialog));
+    dialog = gtk_file_chooser_dialog_new ("Load Magic Card Database",
+			GTK_WINDOW (game->main_window),
+			GTK_FILE_CHOOSER_ACTION_OPEN,
+			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+			GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+			NULL);
+    gtk_signal_connect (GTK_OBJECT (dialog), "response",
+			GTK_SIGNAL_FUNC (find_cardinfo_response), game);
+    gtk_widget_show_all (dialog);
+}
+#else
+static void
+find_cardinfo_ok (GtkWidget *w, GtkWidget *dialog)
+{
+    const char *filename;
     filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (dialog));
+    if (filename != NULL)
+	finish_find_cardinfo (dialog, filename,
+			  gtk_object_get_user_data (GTK_OBJECT (dialog)));
+}
+
+void
+do_find_cardinfo_dialog (struct game *game)
+{
+    GtkWidget *dialog;
+    GtkFileSelection *fsp;
+
+    dialog = gtk_file_selection_new ("Load Magic Card Database");
+    game_make_transient (game, dialog);
+    gtk_object_set_user_data (GTK_OBJECT (dialog), game);
+    fsp = GTK_FILE_SELECTION (dialog);
+    gtk_signal_connect (GTK_OBJECT (fsp->ok_button), "clicked",
+			GTK_SIGNAL_FUNC (find_cardinfo_ok), dialog);
+    gtk_signal_connect_object (GTK_OBJECT (fsp->cancel_button), "clicked",
+			       GTK_SIGNAL_FUNC (gtk_widget_destroy),
+			       GTK_OBJECT (dialog));
+    gtk_widget_show (dialog);
+}
+#endif
+
+static void
+finish_find_cardinfo (GtkWidget *dialog, const char *filename,
+		      struct game *game)
+{
+    char *text;
+    struct cardbase *cardbase;
+    int pid;
+    struct player *me;
+
+    if (game->opponent != NULL) {
+	text = "Cannot load new card data while game is in progress.";
+	do_message_dialog (text);
+	return;
+    }
     if (filename == NULL)
 	return;
+    cardbase = NULL;
     if (is_ascii_file (filename)) {
-	game->cardbase = cardbase_new_from_ort (filename);
-	if (game->cardbase == NULL) {
+	cardbase = cardbase_new_from_ort (filename);
+	if (cardbase == NULL) {
 	    display_message (game, "%s: Failed to read oracle card database.",
 			     filename);
 	    return;
 	}
 	set_preference (game->prefs, oracle_name_key, filename);
+	clear_preference (game->prefs, cardinfo_name_key);
     }
     else {
-	game->cardbase = cardbase_new_from_dat (filename);
-	if (game->cardbase == NULL) {
+	cardbase = cardbase_new_from_dat (filename);
+	if (cardbase == NULL) {
 	    display_message (game, "%s: Failed to read cardinfo database.",
 			     filename);
 	    return;
 	}
 	set_preference (game->prefs, cardinfo_name_key, filename);
+	clear_preference (game->prefs, oracle_name_key);
     }
     gtk_widget_destroy (dialog);
+
+    if (game->cardbase != NULL) {
+	game_empty_all_zones (game);
+	for (pid = 0; pid < game->num_players; pid++) {
+	    me = game->player[pid];
+	    if (me->deck != NULL) {
+		display_message (game, "Unloading %s's deck.", me->name);
+		deck_destroy (me->deck);
+		me->deck = NULL;
+	    }
+	}
+	if (game->expansions != NULL) {
+	    display_message (game, "Unloading expansions data.");
+	    expansions_destroy (game->expansions);
+	    game->expansions = NULL;
+	}
+	cardbase_destroy (game->cardbase);
+    }
+    game->cardbase = cardbase;
 }
 
 /**
