@@ -24,14 +24,17 @@ void do_verify_dialog (struct game *game, const char *title,
 void do_life_dialog (struct game *game, int what_todo);
 void do_library_dialog (struct game *game);
 void do_zone_dialog (struct game *game, int pid, int znum, int qty);
+void do_qzone_dialog (struct game *game, int pid, int znum, int qty,
+		      int private);
 void view_card_selected_in_zone_list (struct game *, int, int);
 int gtk_clist_get_selected_row (GtkCList *clist);
 void do_create_card_dialog (struct game *game);
 void align_and_attach (GtkWidget *, GtkWidget *, int, int, int, int);
 void do_view_card_dialog (struct game *game, int cid);
+void do_view_card_dialog_from_cardinfo (struct game *, struct cardinfo *);
 GtkWidget *new_wide_button (const char *text, int pad);
 void do_preferences_dialog (struct game *game);
-void do_sideboard_dialog (struct game *game);
+void do_sideboard_dialog (struct game *game, int creating);
 void do_find_cardinfo_dialog (struct game *game);
 
 /* mindless.c */
