@@ -16,6 +16,7 @@
 #include <time.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <errno.h>
 #include <gtk/gtk.h>
 #include "game.h"
 #include "cardbase.h"
@@ -25,6 +26,8 @@
 #include "opponent.h"
 #include "prefs.h"
 #include "cardart.h"
+#include "expansions.h"
+#include "dialogs.h"
 
 char *color_names[] = {
     "White", "Blue", "Black", "Red", "Green", "Gold", "Artifact", "Land",
@@ -397,6 +400,14 @@ int game_create_card (struct game *game, int pid, char *name, int idx,
     return cid;
 }
 
+void game_make_transient (struct game *game, GtkWidget *dialog)
+{
+    if (get_int_preference (game->prefs, "dialog_windows", 0) == 0) {
+	gtk_window_set_transient_for (GTK_WINDOW (dialog),
+				      GTK_WINDOW (game->main_window));
+    }
+}
+
 /* ----------------------------------------------------------------- */
 
 void display_message (struct game *game, char *fmt, ...)
@@ -419,7 +430,7 @@ void display_message (struct game *game, char *fmt, ...)
 	GtkTextMark *mark;
 	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(game->message_box));
 	gtk_text_buffer_get_bounds(buffer, &start, &end);
-	gtk_text_buffer_insert (buffer, &end, g_locale_to_utf8(message, -1, NULL, NULL, NULL), -1);
+	gtk_text_buffer_insert (buffer, &end, message, -1);
 	mark = gtk_text_buffer_create_mark(buffer, NULL, &end, FALSE);
 	gtk_text_view_scroll_mark_onscreen(GTK_TEXT_VIEW(game->message_box), mark);
     }
@@ -718,7 +729,7 @@ void game_show_random (struct game *game, int pid, int cid)
 
     pname = game->player[pid]->name;
     cname = game_get_card_name (game, cid);
-    display_message (game, "%s randomly reveals: %s.", pname, cname);
+    display_message (game, "%s randomly shows '%s'", pname, cname);
 }
 
 int color_name_to_number (char *ctext)
@@ -788,3 +799,57 @@ void game_save_deck (struct game *game, int pid)
 	deck_write_file (filename, me->deck, game->cardbase);
     }
 }
+
+/**
+ * Load the expansions data.
+ */
+void game_load_expansions (struct game *game)
+{
+    struct cardbase *cardbase;
+    int count, i;
+    char *filename, *cp;
+    struct expansions *expansions;
+
+    /*
+     * If the cardbase was loaded from an Oracle file,
+     * then it doesn't have expansion data.
+     */
+    cardbase = game->cardbase;
+    count = 0;
+    for (i = 0; i < cardbase->num_cards; i++) {
+	if ((cardbase->cards[i].expansion != NULL) &&
+	    (cardbase->cards[i].expansion[0] != 0))
+	    count++;
+    }
+    if (count <= cardbase->num_cards / 10) {
+	cp = "Cannot create sealed deck because card database\n"
+	    "does not contain expansion data.\n"
+	    "Try an Apprentice CardInfo.dat";
+	do_message_dialog (cp);
+	return;
+    }
+
+    filename = malloc (strlen (cardbase->filename) + 96);
+    strcpy (filename, cardbase->filename);
+    cp = strrchr (filename, '/');
+    cp = (cp == NULL ? filename : cp + 1);
+    strcpy (cp, "Expan.dat");
+    expansions = expansions_read_expandat (filename, cardbase);
+    if (expansions == NULL) {
+	strcat (filename, ": ");
+	strcat (filename, strerror (errno));
+	do_message_dialog (filename);
+    }
+    else {
+	strcpy (cp, "Distro.dat");
+	if (!expansions_read_distrodat (expansions, filename)) {
+	    strcat (filename, ": ");
+	    strcat (filename, strerror (errno));
+	    do_message_dialog (filename);
+	    expansions_destroy (expansions);
+	    expansions = NULL;
+	}
+    }
+    free (filename);
+    game->expansions = expansions;
+}
