@@ -1,6 +1,7 @@
 /* http_fetcher.c - HTTP handling functions
 
-	Copyright (C) 2001 Lyle Hanson (lhanson@cs.nmu.edu)
+	HTTP Fetcher 
+	Copyright (C) 2001, 2003, 2004 Lyle Hanson (lhanson@users.sourceforge.net)
 
 	This library is free software; you can redistribute it and/or
 	modify it under the terms of the GNU Library General Public
@@ -12,20 +13,22 @@
 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 	Library General Public License for more details.
 
-	See included LICENSE file for details
+	See LICENSE file for details
 
  */
 
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
-#include <sys/time.h>
+#include <strings.h>
+#include <ctype.h>
+#include <errno.h>
 #include <netdb.h>
 #include <unistd.h>
-#include <errno.h>
 #include <netinet/in.h>
 #include <sys/types.h>
 #include <sys/socket.h>
+#include <sys/time.h>
 #include "http_fetcher.h"
 
 /* Globals */
@@ -34,8 +37,9 @@ char *userAgent = NULL;
 char *referer = NULL;
 int hideUserAgent = 0;
 int hideReferer = 1;
-const char *http_errlist[];	/* Array of HTTP Fetcher error messages */
-char convertedError[128];	/* Buffer to used when errors contain %d */
+static int followRedirects = DEFAULT_REDIRECTS;	/* # of redirects to follow */
+extern const char *http_errlist[];	/* Array of HTTP Fetcher error messages */
+extern char convertedError[128];	/* Buffer to used when errors contain %d */
 static int errorSource = 0;
 static int http_errno = 0;
 static int errorInt = 0;			/* When the error message has a %d in it,
@@ -52,11 +56,15 @@ int http_fetch(const char *url_tmp, char **fileBuf)
 	{
 	fd_set rfds;
 	struct timeval tv;
-	char requestBuf[REQUEST_BUF_SIZE];
 	char headerBuf[HEADER_BUF_SIZE];
-	char *url, *pageBuf, *host, *charIndex;
-	int sock, bytesRead = 0, contentLength = -1;
-	int ret = -1, i, selectRet;
+	char *tmp, *url, *pageBuf, *requestBuf = NULL, *host, *charIndex;
+	int sock, bytesRead = 0, contentLength = -1, bufsize = REQUEST_BUF_SIZE;
+	int i,
+		ret = -1,
+		tempSize,
+		selectRet,
+		found = 0,	/* For redirects */
+		redirectsFollowed = 0;
 
 
 	if(url_tmp == NULL)
@@ -75,105 +83,276 @@ int http_fetch(const char *url_tmp, char **fileBuf)
 		}
 	strncpy(url, url_tmp, strlen(url_tmp) + 1);
 	
-	/* Seek to the file path portion of the url */
-	charIndex = strstr(url, "://");
-	if(charIndex != NULL)
+	/* This loop allows us to follow redirects if need be.  An afterthought,
+	 * added to provide this basic functionality.  Will hopefully be designed
+	 * better in 2.x.x ;) */
+/*	while(!found &&
+		  (followRedirects < 0 || redirectsFollowed < followRedirects) )
+  */  do
 		{
-		/* url contains a protocol field */
-		charIndex += strlen("://");
-		host = charIndex;
-		charIndex = strchr(charIndex, '/');
-		}
-	else
-		{
-		host = (char *)url;
-		charIndex = strchr(url, '/');
-		}
+		/* Seek to the file path portion of the url */
+		charIndex = strstr(url, "://");
+		if(charIndex != NULL)
+			{
+			/* url contains a protocol field */
+			charIndex += strlen("://");
+			host = charIndex;
+			charIndex = strchr(charIndex, '/');
+			}
+		else
+			{
+			host = (char *)url;
+			charIndex = strchr(url, '/');
+			}
 
-	/* Compose a request string */
-	if(charIndex == NULL)
-		/* The url has no '/' in it, assume the user is making a root-level
-		 *	request */ 
-		sprintf(requestBuf, "GET / %s\n", HTTP_VERSION);
-	else
-		sprintf(requestBuf, "GET %s %s\n", charIndex, HTTP_VERSION);
+		/* Compose a request string */
+		requestBuf = malloc(bufsize);
+		if(requestBuf == NULL)
+			{
+			free(url);
+			errorSource = ERRNO;
+			return -1;
+			}
+		requestBuf[0] = 0;
 
-	/* Null out the end of the hostname if need be */
-	if(charIndex != NULL)
-		*charIndex = 0;
-	/* Use Host: even though 1.0 doesn't specify it.  Some servers
-	 *	won't play nice if we don't send Host, and it shouldn't hurt anything */
-	strcat(requestBuf, "Host: ");
-	strcat(requestBuf, host);
-	strcat(requestBuf, "\n");
+		if(charIndex == NULL)
+			{
+			/* The url has no '/' in it, assume the user is making a root-level
+			 *	request */ 
+			tempSize = strlen("GET /") + strlen(HTTP_VERSION) + 2;
+			if(_checkBufSize(&requestBuf, &bufsize, tempSize) ||
+				snprintf(requestBuf, bufsize, "GET / %s\r\n", HTTP_VERSION) < 0)
+				{
+				free(url);
+				free(requestBuf);
+				errorSource = ERRNO;
+				return -1;
+				}
+			}
+		else
+			{
+			tempSize = strlen("GET ") + strlen(charIndex) +
+  	          strlen(HTTP_VERSION) + 4;
+		 	/* + 4 is for ' ', '\r', '\n', and NULL */
+                                    
+			if(_checkBufSize(&requestBuf, &bufsize, tempSize) ||
+					snprintf(requestBuf, bufsize, "GET %s %s\r\n",
+					charIndex, HTTP_VERSION) < 0)
+				{
+				free(url);
+				free(requestBuf);
+				errorSource = ERRNO;
+				return -1;
+				}
+			}
 
-	if(!hideReferer && referer != NULL)	/* NO default referer */
-		{
-		strcat(requestBuf, "Referer: ");
-		strcat(requestBuf, referer);
-		strcat(requestBuf, "\n");
-		}
+		/* Null out the end of the hostname if need be */
+		if(charIndex != NULL)
+			*charIndex = 0;
 
-	if(!hideUserAgent && userAgent == NULL)
-		{
-		strcat(requestBuf, "User-Agent: ");
-		strcat(requestBuf, DEFAULT_USER_AGENT);
-		strcat(requestBuf, "/");
-		strcat(requestBuf, VERSION);
-		strcat(requestBuf, "\n");
-		}
-	else if(!hideUserAgent)
-		{
-		strcat(requestBuf, "User-Agent: ");
-		strcat(requestBuf, userAgent);
-		strcat(requestBuf, "\n");
-		}
+		/* Use Host: even though 1.0 doesn't specify it.  Some servers
+		 *	won't play nice if we don't send Host, and it shouldn't
+		 *	hurt anything */
+		ret = bufsize - strlen(requestBuf); /* Space left in buffer */
+		tempSize = (int)strlen("Host: ") + (int)strlen(host) + 3;
+        /* +3 for "\r\n\0" */
+		if(_checkBufSize(&requestBuf, &bufsize, tempSize + 128))
+			{
+			free(url);
+			free(requestBuf);
+			errorSource = ERRNO;
+			return -1;
+			}
+		strcat(requestBuf, "Host: ");
+		strcat(requestBuf, host);
+		strcat(requestBuf, "\r\n");
 
-	strcat(requestBuf, "Connection: Close\n\n");
+		if(!hideReferer && referer != NULL)	/* NO default referer */
+			{
+			tempSize = (int)strlen("Referer: ") + (int)strlen(referer) + 3;
+   	        /* + 3 is for '\r', '\n', and NULL */
+			if(_checkBufSize(&requestBuf, &bufsize, tempSize))
+				{
+				free(url);
+				free(requestBuf);
+				errorSource = ERRNO;
+				return -1;
+				}
+			strcat(requestBuf, "Referer: ");
+			strcat(requestBuf, referer);
+			strcat(requestBuf, "\r\n");
+			}
 
+		if(!hideUserAgent && userAgent == NULL)
+			{
+			tempSize = (int)strlen("User-Agent: ") +
+				(int)strlen(DEFAULT_USER_AGENT) + (int)strlen(VERSION) + 4;
+   	        /* + 4 is for '\', '\r', '\n', and NULL */
+			if(_checkBufSize(&requestBuf, &bufsize, tempSize))
+				{
+				free(url);
+				free(requestBuf);
+				errorSource = ERRNO;
+				return -1;
+				}
+			strcat(requestBuf, "User-Agent: ");
+			strcat(requestBuf, DEFAULT_USER_AGENT);
+			strcat(requestBuf, "/");
+			strcat(requestBuf, VERSION);
+			strcat(requestBuf, "\r\n");
+			}
+		else if(!hideUserAgent)
+			{
+			tempSize = (int)strlen("User-Agent: ") + (int)strlen(userAgent) + 3;
+   	        /* + 3 is for '\r', '\n', and NULL */
+			if(_checkBufSize(&requestBuf, &bufsize, tempSize))
+				{
+				free(url);
+				free(requestBuf);
+				errorSource = ERRNO;
+				return -1;
+				}
+			strcat(requestBuf, "User-Agent: ");
+			strcat(requestBuf, userAgent);
+			strcat(requestBuf, "\r\n");
+			}
 
-	sock = makeSocket(host);			/* errorSource set within makeSocket */
-	if(sock == -1) { free(url); return -1;}
+		tempSize = (int)strlen("Connection: Close\r\n\r\n");
+		if(_checkBufSize(&requestBuf, &bufsize, tempSize))
+			{
+			free(url);
+			free(requestBuf);
+			errorSource = ERRNO;
+			return -1;
+			}
+		strcat(requestBuf, "Connection: Close\r\n\r\n");
 
-	if(write(sock, requestBuf, strlen(requestBuf)) == -1)
-		{ close(sock); free(url); errorSource = ERRNO; return -1; }
+		/* Now free any excess memory allocated to the buffer */
+		tmp = realloc(requestBuf, strlen(requestBuf) + 1);
+		if(tmp == NULL)
+			{
+			free(url);
+			free(requestBuf);
+			errorSource = ERRNO;
+			return -1;
+			}
+		requestBuf = tmp;
 
-	/* Grab enough of the response to get the metadata */
-	ret = _http_read_header(sock, headerBuf);	/* errorSource set within */
-	if(ret < 0) { close(sock); free(url); return -1; }
+		sock = makeSocket(host);		/* errorSource set within makeSocket */
+		if(sock == -1) { free(url); free(requestBuf); return -1;}
 
-	/* Get the return code */
-	charIndex = strstr(headerBuf, "HTTP/");
-	if(charIndex == NULL)
-		{
-		close(sock);
 		free(url);
-		errorSource = FETCHER_ERROR;
-		http_errno = HF_FRETURNCODE;
-		return -1;
-		}
-	while(*charIndex != ' ')
+        url = NULL;
+
+		if(write(sock, requestBuf, strlen(requestBuf)) == -1)
+			{
+			close(sock);
+			free(requestBuf);
+			errorSource = ERRNO;
+			return -1;
+			}
+
+		free(requestBuf);
+        requestBuf = NULL;
+
+		/* Grab enough of the response to get the metadata */
+		ret = _http_read_header(sock, headerBuf);	/* errorSource set within */
+		if(ret < 0) { close(sock); return -1; }
+
+		/* Get the return code */
+		charIndex = strstr(headerBuf, "HTTP/");
+		if(charIndex == NULL)
+			{
+			close(sock);
+			errorSource = FETCHER_ERROR;
+			http_errno = HF_FRETURNCODE;
+			return -1;
+			}
+		while(*charIndex != ' ')
+			charIndex++;
 		charIndex++;
-	charIndex++;
 
-	ret = sscanf(charIndex, "%i", &i);
-	if(ret != 1)
-		{
-		close(sock);
-		free(url);
-		errorSource = FETCHER_ERROR;
-		http_errno = HF_CRETURNCODE;
-		return -1;
-		}
-	if(i<200 || i>299)
-		{
-		close(sock);
-		free(url);
-		errorInt = i;	/* Status code, to be inserted in error string */
-		errorSource = FETCHER_ERROR;
-		http_errno = HF_STATUSCODE;
-		return -1;
-		}
+		ret = sscanf(charIndex, "%d", &i);
+		if(ret != 1)
+			{
+			close(sock);
+			errorSource = FETCHER_ERROR;
+			http_errno = HF_CRETURNCODE;
+			return -1;
+			}
+		if(i<200 || i>307)
+			{
+			close(sock);
+			errorInt = i;	/* Status code, to be inserted in error string */
+			errorSource = FETCHER_ERROR;
+			http_errno = HF_STATUSCODE;
+			return -1;
+			}
+
+		/* If a redirect, repeat operation until final URL is found or we
+		 *  redirect followRedirects times.  Note the case sensitive "Location",
+		 *  should probably be made more robust in the future (without relying
+		 *  on the non-standard strcasecmp()).
+		 * This bit mostly by Dean Wilder, tweaked by me */
+		if(i >= 300)
+			{
+		    redirectsFollowed++;
+
+			/* Pick up redirect URL, allocate new url, and repeat process */
+			charIndex = strstr(headerBuf, "Location:");
+			if(!charIndex)
+				{
+				close(sock);
+				errorInt = i; /* Status code, to be inserted in error string */
+				errorSource = FETCHER_ERROR;
+				http_errno = HF_CANTREDIRECT;
+				return -1;
+				}
+			charIndex += strlen("Location:");
+            /* Skip any whitespace... */
+            while(*charIndex != '\0' && isspace(*charIndex))
+                charIndex++;
+            if(*charIndex == '\0')
+                {
+				close(sock);
+				errorInt = i; /* Status code, to be inserted in error string */
+				errorSource = FETCHER_ERROR;
+				http_errno = HF_CANTREDIRECT;
+				return -1;
+                }
+
+			i = strcspn(charIndex, " \r\n");
+			if(i > 0)
+				{
+				url = (char *)malloc(i + 1);
+				strncpy(url, charIndex, i);
+				url[i] = '\0';
+				}
+			else
+                /* Found 'Location:' but contains no URL!  We'll handle it as
+                 * 'found', hopefully the resulting document will give the user
+                 * a hint as to what happened. */
+                found = 1;
+            }
+		else
+			found = 1;
+	    } while(!found &&
+                (followRedirects < 0 || redirectsFollowed <= followRedirects) );
+
+    if(url) /* Redirection code may malloc this, then exceed followRedirects */
+        {
+        free(url);
+        url = NULL;
+        }
+    
+    if(redirectsFollowed >= followRedirects && !found)
+        {
+        close(sock);
+    	errorInt = followRedirects; /* To be inserted in error string */
+    	errorSource = FETCHER_ERROR;
+    	http_errno = HF_MAXREDIRECTS;
+	    return -1;
+        }
 	
 	/*
 	 * Parse out about how big the data segment is.
@@ -189,12 +368,11 @@ int http_fetch(const char *url_tmp, char **fileBuf)
 
 	if(charIndex != NULL)
 		{
-		ret = sscanf(charIndex + strlen("content-length: "), "%i",
+		ret = sscanf(charIndex + strlen("content-length: "), "%d",
 			&contentLength);
 		if(ret < 1)
 			{
 			close(sock);
-			free(url);
 			errorSource = FETCHER_ERROR;
 			http_errno = HF_CONTENTLEN;
 			return -1;
@@ -209,7 +387,6 @@ int http_fetch(const char *url_tmp, char **fileBuf)
 	if(pageBuf == NULL)
 		{
 		close(sock);
-		free(url);
 		errorSource = ERRNO;
 		return -1;
 		}
@@ -227,20 +404,18 @@ int http_fetch(const char *url_tmp, char **fileBuf)
 		else		/* No timeout, can block indefinately */
 			selectRet = select(sock+1, &rfds, NULL, NULL, NULL);
 
-		if(selectRet == 0 && timeout < 0)
+		if(selectRet == 0)
 			{
 			errorSource = FETCHER_ERROR;
 			http_errno = HF_DATATIMEOUT;
 			errorInt = timeout;
 			close(sock);
-			free(url);
 			free(pageBuf);
 			return -1;
 			}
 		else if(selectRet == -1)
 			{
 			close(sock);
-			free(url);
 			free(pageBuf);
 			errorSource = ERRNO;
 			return -1;
@@ -250,7 +425,6 @@ int http_fetch(const char *url_tmp, char **fileBuf)
 		if(ret == -1)
 			{
 			close(sock);
-			free(url);
 			free(pageBuf);
 			errorSource = ERRNO;
 			return -1;
@@ -264,34 +438,38 @@ int http_fetch(const char *url_tmp, char **fileBuf)
 			 *	allocate another read-sized chunk to make sure we have
 			 *	enough room.
 			 */
-			pageBuf = (char *)realloc(pageBuf, bytesRead + contentLength);
-			if(pageBuf == NULL)
+			tmp = (char *)realloc(pageBuf, bytesRead + contentLength);
+			if(tmp == NULL)
 				{
 				close(sock);
-				free(url);
 				free(pageBuf);
 				errorSource = ERRNO;
 				return -1;
 				}
+            pageBuf = tmp;
 			}
 		}
 	
 	/*
 	 * The download buffer is too large.  Trim off the safety padding.
+     * Note that we add one NULL byte to the end of the data, as it may not
+     *  already be NULL terminated and we can't be sure what type of data it
+     *  is or what the caller will do with it.
 	 */
-	pageBuf = (char *)realloc(pageBuf, bytesRead);
-		/* pageBuf shouldn't be null, since we're _shrinking_ the buffer,
+	tmp = (char *)realloc(pageBuf, bytesRead + 1);
+		/* tmp shouldn't be null, since we're _shrinking_ the buffer,
 		 *	and if it DID fail, we could go on with the too-large buffer,
 		 *	but something would DEFINATELY be wrong, so we'll just give
 		 *	an error message */
-	if(pageBuf == NULL)
+	if(tmp == NULL)
 		{
 		close(sock);
-		free(url);
 		free(pageBuf);
 		errorSource = ERRNO;
 		return -1;
 		}
+    pageBuf = tmp;
+    pageBuf[bytesRead] = '\0';  /* NULL terminate the data */
 
 	if(fileBuf == NULL)	/* They just wanted us to "hit" the url */
 		free(pageBuf);
@@ -299,7 +477,6 @@ int http_fetch(const char *url_tmp, char **fileBuf)
 		*fileBuf = pageBuf;
 
 	close(sock);
-	free(url);
 	return bytesRead;
 	}
 
@@ -373,6 +550,21 @@ void http_setTimeout(int seconds) { timeout = seconds; }
 
 
 
+	/*
+	 * Changes the number of HTTP redirects HTTP Fetcher will automatically
+	 *	follow.  If a request returns a status code of 3XX and contains
+	 *	a "Location:" field, the library will transparently follow up to
+	 *	the specified number of redirects.  With this implementation
+	 *	(which is just a stopgap, really) the caller won't be aware of any
+	 *	redirection and will assume the returned document came from the original
+	 *	URL.
+	 * To disable redirects, pass a 0.  To follow unlimited redirects (probably
+	 *  unwise), pass a negative value.  The default is to follow 3 redirects.
+	 */
+void http_setRedirects(int redirects) { followRedirects = redirects; }
+
+
+
 	/*
 	 * Puts the filename portion of the url into 'filename'.
 	 * Returns:
@@ -392,7 +584,7 @@ int http_parseFilename(const char *url, char **filename)
 		return -1;
 		}
 
-	ptr = (char *)strrchr(url, '/');
+	ptr = (char *)rindex(url, '/');
 	if(ptr == NULL)
 		/* Root level request, apparently */
 		return 1;
@@ -459,10 +651,16 @@ void http_perror(const char *string)
 	 */
 const char *http_strerror()
 	{
+	extern int errno;
+	
 	if(errorSource == ERRNO)
 		return strerror(errno);
 	else if(errorSource == H_ERRNO)
+#ifdef HAVE_HSTRERROR
 		return hstrerror(h_errno);
+#else
+		return http_errlist[HF_HERROR];
+#endif
 	else if(errorSource == FETCHER_ERROR)
 		{
 		if(strstr(http_errlist[http_errno], "%d") == NULL)
@@ -516,7 +714,7 @@ int _http_read_header(int sock, char *headerPtr)
 		else		/* No timeout, can block indefinately */
 			selectRet = select(sock+1, &rfds, NULL, NULL, NULL);
 		
-		if(selectRet == 0 && timeout < 0)
+		if(selectRet == 0)
 			{
 			errorSource = FETCHER_ERROR;
 			http_errno = HF_HEADTIMEOUT;
@@ -549,8 +747,8 @@ int _http_read_header(int sock, char *headerPtr)
 	return bytesRead;
 	}
 
-	
-	
+
+
 	/*
 	 * Opens a TCP socket and returns the descriptor
 	 * Returns:
@@ -563,14 +761,26 @@ int makeSocket(const char *host)
 	struct sockaddr_in sa;							/* Socket address */
 	struct hostent *hp;								/* Host entity */
 	int ret;
+    int port;
+    char *p;
 	
+    /* Check for port number specified in URL */
+    p = strchr(host, ':');
+    if(p)
+        {
+        port = atoi(p + 1);
+        *p = '\0';
+        }
+    else
+        port = PORT_NUMBER;
+
 	hp = gethostbyname(host);
 	if(hp == NULL) { errorSource = H_ERRNO; return -1; }
 		
 	/* Copy host address from hostent to (server) socket address */
 	memcpy((char *)&sa.sin_addr, (char *)hp->h_addr, hp->h_length);
 	sa.sin_family = hp->h_addrtype;		/* Set service sin_family to PF_INET */
-	sa.sin_port = htons(PORT_NUMBER);	/* Put portnum into sockaddr */
+	sa.sin_port = htons(port);      	/* Put portnum into sockaddr */
 
 	sock = socket(hp->h_addrtype, SOCK_STREAM, 0);
 	if(sock == -1) { errorSource = ERRNO; return -1; }
@@ -582,6 +792,31 @@ int makeSocket(const char *host)
 	}
 
 
+
+	/*
+	 * Determines if the given NULL-terminated buffer is large enough to
+	 * 	concatenate the given number of characters.  If not, it attempts to
+	 * 	grow the buffer to fit.
+	 * Returns:
+	 *	0 on success, or
+	 *	-1 on error (original buffer is unchanged).
+	 */
+int _checkBufSize(char **buf, int *bufsize, int more)
+	{
+	char *tmp;
+	int roomLeft = *bufsize - (strlen(*buf) + 1);
+	if(roomLeft > more)
+		return 0;
+	tmp = realloc(*buf, *bufsize + more + 1);
+	if(tmp == NULL)
+		return -1;
+	*buf = tmp;
+	*bufsize += more + 1;
+	return 0;
+	}
+
+
+	/* Note that '%d' cannot be escaped at this time */
 const char *http_errlist[] =
 	{
 	"Success",										/* HF_SUCCESS		*/
@@ -592,5 +827,12 @@ const char *http_errlist[] =
 	"Couldn't find return code in HTTP response",	/* HF_FRETURNCODE	*/
 	"Couldn't convert return code in HTTP response",/* HF_CRETURNCODE	*/
 	"Request returned a status code of %d",			/* HF_STATUSCODE	*/
-	"Couldn't convert Content-Length to integer"	/* HF_CONTENTLEN	*/
+	"Couldn't convert Content-Length to integer",	/* HF_CONTENTLEN	*/
+	"Network error (description unavailable)",		/* HF_HERROR		*/
+	"Status code of %d but no Location: field",		/* HF_CANTREDIRECT  */
+	"Followed the maximum number of redirects (%d)" /* HF_MAXREDIRECTS  */
 	};
+
+	/* Used to copy in messages from http_errlist[] and replace %d's with
+	 *	the value of errorInt.  Then we can pass the pointer to THIS */
+char convertedError[128];
