@@ -24,11 +24,13 @@
 #include "prefs.h"
 #include "random.h"
 #include "version.h"
+#include "images.h"
 
 static void add_tools (GtkToolbar *toolbar);
-static void add_phase_button (GtkToolbar *, char *, char *, int);
+static void add_phase_button (GtkToolbar *, char *, char *, int, const char *);
 static void add_drop_tools (GtkToolbar *toolbar);
 static void chdir_of_file (char *filename);
+static void do_load_cardbase (GtkWidget *w, gpointer data);
 static void do_new_game (GtkWidget *w, gpointer data);
 static void do_disconnect (GtkWidget *w, gpointer data);
 static void do_exit (GtkWidget *w, gpointer data);
@@ -42,6 +44,7 @@ static void do_shuffle (GtkWidget *w, gpointer data);
 static void do_mulligan (GtkWidget *w, gpointer data);
 static void do_grave_to_lib (GtkWidget *w, gpointer data);
 static void do_grave_to_rem (GtkWidget *w, gpointer data);
+static void do_toplib_to_grave (GtkWidget *w, gpointer data);
 static void move_cards_in_zone (struct game *game, int src, int dest);
 static void do_set_phase (GtkWidget *w, gpointer data);
 static void do_next_turn (GtkWidget *w, gpointer data);
@@ -79,6 +82,9 @@ static void set_main_window_size (struct game *game, GtkWidget *);
 #define MICON(x) NULL
 #define KEY_EVENT "key_press_event"
 #endif
+#ifndef GTK_STOCK_ABOUT
+#define GTK_STOCK_ABOUT GTK_STOCK_DIALOG_INFO
+#endif
 
 /*
  *  When I get the time, I'll make this global variable go away.
@@ -87,36 +93,38 @@ struct game *game;
 
 static GtkItemFactoryEntry menu_items[] = {
     { "/_File", NULL, NULL, 0, "<Branch>" },
-    { "/File/_New Game...", "", do_new_game, 0, MICON(GTK_STOCK_INDEX) },
+    { "/File/_New Game...", "", do_new_game, 0, MICON(M_NEW_ICON) },
+    { "/File/_Load Card Data...", "", do_load_cardbase, 0, MICON(GTK_STOCK_OPEN) },
     { "/File/sep1", NULL, NULL, 0, "<Separator>" },
     { "/File/_Disconnect", "", do_disconnect, 0, MICON(GTK_STOCK_STOP) },
     { "/File/sep2", NULL, NULL, 0, "<Separator>" },
     { "/File/_Quit", "", do_exit, 0, MICON(GTK_STOCK_QUIT) },
 
     { "/_Game", NULL, NULL, 0, "<Branch>" },
-    { "/Game/Set _Life...", "<control>L", do_set_life, SET_LIFE, MICON(GTK_STOCK_JUMP_TO) },
+    { "/Game/Set _Life...", "<control>L", do_set_life, SET_LIFE, MICON(M_LIFESET_ICON) },
     { "/Game/Next _Phase", "<control>P", do_next_phase, 0, MICON(GTK_STOCK_GO_FORWARD) },
     { "/Game/sep1", NULL, NULL, 0, "<Separator>" },
-    { "/Game/Increment Life", "F12", do_set_life, INCREMENT_LIFE, MICON(GTK_STOCK_ADD) },
-    { "/Game/Decrement Life", "F11", do_set_life, DECREMENT_LIFE, MICON(GTK_STOCK_REMOVE) },
+    { "/Game/Increment Life", "F12", do_set_life, INCREMENT_LIFE, MICON(M_LIFEPLUS_ICON) },
+    { "/Game/Decrement Life", "F11", do_set_life, DECREMENT_LIFE, MICON(M_LIFEMINUS_ICON) },
     { "/Game/sep2", NULL, NULL, 0, "<Separator>" },
     { "/Game/Increment Turn", "<control>T", do_next_turn, 0, MICON(GTK_STOCK_GOTO_LAST) },
     { "/Game/Decrement Turn", "", do_prev_turn, 0, MICON(GTK_STOCK_GOTO_FIRST) },
 
     { "/_Action", NULL, NULL, 0, "<Branch>" },
-    { "/Action/_Draw Card", "<control>D", do_draw_card, 0, MICON(GTK_STOCK_EXECUTE) },
-    { "/Action/_Untap My Permanents", "<control>U", do_untap_cards, 0, MICON(GTK_STOCK_UNDO) },
+    { "/Action/_Draw Card", "<control>D", do_draw_card, 0, MICON(M_DRAW_ICON) },
+    { "/Action/_Untap My Permanents", "<control>U", do_untap_cards, 0, MICON(M_UNTAP_ICON) },
     { "/Action/sep1", NULL, NULL, 0, "<Separator>" },
-    { "/Action/_Create Card...", "", do_create_card, 0, MICON(GTK_STOCK_NEW) },
+    { "/Action/_Create Card...", "", do_create_card, 0, MICON(M_CREATE_ICON) },
     { "/Action/sep2", NULL, NULL, 0, "<Separator>" },
-    { "/Action/_Flip a Coin", "<control>F", do_flip_coin, 0, MICON(GTK_STOCK_DIALOG_QUESTION) },
-    { "/Action/Roll a _Die", "<control>I", do_roll_die, 0, MICON(GTK_STOCK_DIALOG_QUESTION) },
+    { "/Action/_Flip a Coin", "<control>F", do_flip_coin, 0, MICON(M_COIN_ICON) },
+    { "/Action/Roll a _Die...", "<control>I", do_roll_die, 0, MICON(M_DIE_ICON) },
     { "/Action/sep3", NULL, NULL, 0, "<Separator>" },
-    { "/Action/_Shuffle Library", "<control>S", do_shuffle, 0, MICON(GTK_STOCK_REFRESH) },
+    { "/Action/_Shuffle Library", "<control>S", do_shuffle, 0, MICON(M_SHUFFLE_ICON) },
     { "/Action/sep4", NULL, NULL, 0, "<Separator>" },
-    { "/Action/Move Hand to Library", "", do_mulligan, 0, MICON(GTK_STOCK_CONVERT) },
-    { "/Action/Move Graveyard to Library", "", do_grave_to_lib, 0, MICON(GTK_STOCK_UNDELETE) },
-    { "/Action/Move Graveyard to Removed", "", do_grave_to_rem, 0, MICON(GTK_STOCK_DELETE) },
+    { "/Action/Move Hand to Library", "", do_mulligan, 0, NULL },
+    { "/Action/Move Graveyard to Library", "", do_grave_to_lib, 0, NULL },
+    { "/Action/Move Graveyard to Removed", "", do_grave_to_rem, 0, NULL },
+    { "/Action/Move Top of Library to Graveyard", "<control>M", do_toplib_to_grave, 0, NULL },
 
     { "/_View", NULL, NULL, 0, "<Branch>" },
     { "/View/My _Graveyard...", "", do_graveyard, 0, MICON(GTK_STOCK_ZOOM_IN) },
@@ -128,22 +136,22 @@ static GtkItemFactoryEntry menu_items[] = {
     { "/View/My _Library...", "", do_view_library, 0, MICON(GTK_STOCK_ZOOM_IN) },
     { "/View/_Top Cards of My Library...", "", do_view_libtop, 0, MICON(GTK_STOCK_ZOOM_FIT) },
     { "/View/sep3", NULL, NULL, 0, "<Separator>" },
-    { "/View/My Sideboard", "", do_sideboard, 0, NULL },
+    { "/View/My Sideboard", "", do_sideboard, 0, MICON(M_SIDEBOARD_ICON) },
 
     { "/_Show", NULL, NULL, 0, "<Branch>" },
-    { "/Show/_My Hand", "", do_show_hand, 0, MICON(GTK_STOCK_COPY) },
-    { "/Show/My Entire Library", "", do_show_library, 0, MICON(GTK_STOCK_DND_MULTIPLE) },
+    { "/Show/_My Hand", "", do_show_hand, 0, MICON(M_HAND_ICON) },
+    { "/Show/My Entire Library", "", do_show_library, 0, MICON(M_DECK_ICON) },
     { "/Show/sep1", NULL, NULL, 0, "<Separator>" },
     { "/Show/_Random Card From Hand", "", do_show_random, 0, MICON(GTK_STOCK_DIALOG_QUESTION) },
     { "/Show/sep2", NULL, NULL, 0, "<Separator>" },
-    { "/Show/_Top Cards of My Library...", "", do_show_libtop, 0, MICON(GTK_STOCK_JUSTIFY_FILL) },
+    { "/Show/_Top Cards of My Library...", "", do_show_libtop, 0, MICON(M_LIBTOP_ICON) },
 
     { "/_Options", NULL, NULL, 0, "<Branch>" },
     { "/Options/_Preferences...", "", do_preferences, 0, MICON(GTK_STOCK_PREFERENCES) },
     { "/Options/_Set Name...", "", do_set_name, 0, MICON(GTK_STOCK_PROPERTIES) },
 
     { "/_Help", NULL, NULL, 0, "<Branch>" },
-    { "/Help/_About...", "", do_about_box, 0, MICON(GTK_STOCK_DIALOG_INFO) },
+    { "/Help/_About...", "", do_about_box, 0, MICON(GTK_STOCK_ABOUT) },
 };
 
 int main (int argc, char *argv[])
@@ -168,6 +176,7 @@ int main (int argc, char *argv[])
     gtk_init (&argc, &argv);
     gdk_rgb_init ();
     wp = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+    add_stock_images (wp);
     gtk_window_set_title (GTK_WINDOW (wp), "Mindless Automaton");
     gtk_signal_connect (GTK_OBJECT (wp), "delete_event",
 			GTK_SIGNAL_FUNC (do_exit), NULL);
@@ -336,55 +345,65 @@ static void add_tools (GtkToolbar *toolbar)
     GtkIconSize size = gtk_toolbar_get_icon_size (toolbar);
     GtkWidget *image;
 
-    image = gtk_image_new_from_stock (GTK_STOCK_INDEX, size);
+    image = gtk_image_new_from_stock (M_NEW_ICON, size);
     gtk_toolbar_append_item (toolbar, "New", "New Game", NULL, image,
 			     GTK_SIGNAL_FUNC (do_new_game), game);
-    image = gtk_image_new_from_stock (GTK_STOCK_UNDO, size);
+    image = gtk_image_new_from_stock (M_UNTAP_ICON, size);
     gtk_toolbar_append_item (toolbar, "Untap", "Untap Cards", NULL, image,
 			     GTK_SIGNAL_FUNC (do_untap_cards), game);
-    image = gtk_image_new_from_stock (GTK_STOCK_EXECUTE, size);
+    image = gtk_image_new_from_stock (M_DRAW_ICON, size);
     gtk_toolbar_append_item (toolbar, "Draw", "Draw Card", NULL, image,
 			     GTK_SIGNAL_FUNC (do_draw_card), game);
 
     gtk_toolbar_append_space (toolbar);
 
-    add_phase_button (toolbar, "Untap", "Untap Phase", UNTAP_PHASE);
-    add_phase_button (toolbar, "Upkeep", "Upkeep Phase", UPKEEP_PHASE);
-    add_phase_button (toolbar, "Draw", "Draw Step", DRAW_PHASE);
-    add_phase_button (toolbar, "Main", "Main Phase", MAIN_PHASE);
-    add_phase_button (toolbar, "Combat", "Combat Phase", COMBAT_PHASE);
-    add_phase_button (toolbar, "Cleanup", "Cleanup Phase", CLEANUP_PHASE);
+    add_phase_button (toolbar, "Untap", "Untap Phase", UNTAP_PHASE,
+		      M_UNTAP_PHASE_ICON);
+    add_phase_button (toolbar, "Upkeep", "Upkeep Phase", UPKEEP_PHASE,
+		      M_UPKEEP_PHASE_ICON);
+    add_phase_button (toolbar, "Draw", "Draw Step", DRAW_PHASE,
+		      M_DRAW_PHASE_ICON);
+    add_phase_button (toolbar, "Main", "Main Phase", MAIN_PHASE,
+		      M_MAIN_PHASE_ICON);
+    add_phase_button (toolbar, "Combat", "Combat Phase", COMBAT_PHASE,
+		      M_COMBAT_PHASE_ICON);
+    add_phase_button (toolbar, "Cleanup", "Cleanup Phase", CLEANUP_PHASE,
+		      M_CLEANUP_PHASE_ICON);
 
     gtk_toolbar_append_space (toolbar);
 
     image = gtk_image_new_from_stock (GTK_STOCK_GOTO_LAST, size);
     gtk_toolbar_append_item (toolbar, "Next", "Next Turn", NULL, image,
 			     GTK_SIGNAL_FUNC (do_next_turn), game);
-    image = gtk_image_new_from_stock (GTK_STOCK_JUMP_TO, size);
+    image = gtk_image_new_from_stock (M_LIFESET_ICON, size);
     gtk_toolbar_append_item (toolbar, "Life", "Set Life", NULL, image,
 			     GTK_SIGNAL_FUNC (do_set_life),
-			     (gpointer) SET_LIFE);
-    image = gtk_image_new_from_stock (GTK_STOCK_REMOVE, size);
+			     GINT_TO_POINTER (SET_LIFE));
+    image = gtk_image_new_from_stock (M_LIFEMINUS_ICON, size);
     gtk_toolbar_append_item (toolbar, "-", "Decrement Life", NULL, image,
 			     GTK_SIGNAL_FUNC (do_set_life),
-			     (gpointer) DECREMENT_LIFE);
-    image = gtk_image_new_from_stock (GTK_STOCK_ADD, size);
+			     GINT_TO_POINTER (DECREMENT_LIFE));
+    image = gtk_image_new_from_stock (M_LIFEPLUS_ICON, size);
     gtk_toolbar_append_item (toolbar, "+", "Increment Life", NULL, image,
 			     GTK_SIGNAL_FUNC (do_set_life),
-			     (gpointer) INCREMENT_LIFE);
-    image = gtk_image_new_from_stock (GTK_STOCK_DIALOG_INFO, size);
+			     GINT_TO_POINTER (INCREMENT_LIFE));
+    image = gtk_image_new_from_stock (M_PING_ICON, size);
     gtk_toolbar_append_item (toolbar, "Ping", "Ping Opponent", NULL, image,
 			     GTK_SIGNAL_FUNC (do_ping_opponent), game);
 }
 
 static void add_phase_button (GtkToolbar *toolbar, char *text, char *tip,
-			      int phase)
+			      int phase, const char *icon)
 {
+    GtkIconSize size = gtk_toolbar_get_icon_size (toolbar);
+    GtkWidget *image;
+
+    image = gtk_image_new_from_stock (icon, size);
     game->phase_button[phase] =
 	gtk_toolbar_append_element (toolbar, GTK_TOOLBAR_CHILD_TOGGLEBUTTON,
-				    NULL, text, tip, NULL, NULL,
+				    NULL, text, tip, NULL, image,
 				    GTK_SIGNAL_FUNC(do_set_phase),
-				    (gpointer) phase);
+				    GINT_TO_POINTER(phase));
 }
 
 static void add_drop_tools (GtkToolbar *toolbar)
@@ -393,15 +412,15 @@ static void add_drop_tools (GtkToolbar *toolbar)
     GtkIconSize size = gtk_toolbar_get_icon_size (toolbar);
     struct player *me;
 
-    image = gtk_image_new_from_stock (GTK_STOCK_DND_MULTIPLE, size);
+    image = gtk_image_new_from_stock (M_DECK_ICON, size);
     t1 = gtk_toolbar_append_item (toolbar, "Library", "Library",
 				  NULL, image,
 				  GTK_SIGNAL_FUNC (do_library), game);
-    image = gtk_image_new_from_stock (GTK_STOCK_UNDELETE, size);
+    image = gtk_image_new_from_stock (M_GRAVEYARD_ICON, size);
     t2 = gtk_toolbar_append_item (toolbar, "Graveyard", "Graveyard",
 				  NULL, image,
 				  GTK_SIGNAL_FUNC (do_graveyard), game);
-    image = gtk_image_new_from_stock (GTK_STOCK_DELETE, size);
+    image = gtk_image_new_from_stock (M_REMOVED_ICON, size);
     t3 = gtk_toolbar_append_item (toolbar, "Removed", "Remove From Game",
 				  NULL, image,
 				  GTK_SIGNAL_FUNC (do_removed), game);
@@ -425,6 +444,18 @@ static void chdir_of_file (char *filename)
     free (directory);
 }
 
+static void do_load_cardbase (GtkWidget *w, gpointer data)
+{
+    char *text;
+
+    if (game->opponent != NULL) {
+	text = "Cannot load new card data while game is in progress.";
+	do_message_dialog (text);
+    } else {
+	do_find_cardinfo_dialog (game);
+    }
+}
+
 static void do_new_game (GtkWidget *w, gpointer data)
 {
     if (game->cardbase == NULL) {
@@ -561,6 +592,23 @@ static void do_grave_to_rem (GtkWidget *w, gpointer data)
     move_cards_in_zone (game, GRAVEYARD_ZONE, REMOVED_ZONE);
 }
 
+static void do_toplib_to_grave (GtkWidget *w, gpointer data)
+{
+    struct card_move m;
+
+    m.pid = game->local_player;
+    m.srcznum = LIBRARY_ZONE;
+    m.idx = -1;
+    m.dstznum = GRAVEYARD_ZONE;
+    m.bottom = FALSE;
+    m.cid = 0;
+    game_move_card (game, &m);
+    if (m.cid > 0) {
+	game_move_card_message (game, &m);
+	send_card_zone (game, &m);
+    }
+}
+
 static void move_cards_in_zone (struct game *game, int srcznum, int dstznum)
 {
     struct card_move m;
@@ -582,7 +630,7 @@ static void move_cards_in_zone (struct game *game, int srcznum, int dstznum)
 
 static void do_set_phase (GtkWidget *widget, gpointer data)
 {
-    int new_phase = (int) data;
+    int new_phase = GPOINTER_TO_INT (data);
     GtkToggleButton *button;
 
     if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
@@ -622,7 +670,7 @@ static void do_set_life (GtkWidget *w, gpointer data)
     int todo, offset;
     struct player *me;
 
-    todo = (int) data;
+    todo = GPOINTER_TO_INT (data);
     offset = 0;
 
     switch (todo) {
