@@ -21,6 +21,7 @@
 #include "opponent.h"
 #include "prefs.h"
 #include "dialogs.h"
+#include "sealed.h"
 
 struct newgame_args {
     struct game *game;
@@ -40,7 +41,9 @@ struct newgame_args {
 };
 
 static void do_load_deck (GtkWidget *w, struct newgame_args *args);
-static void file_ok_sel (GtkWidget *w, gpointer data);
+static void finish_load_deck (GtkWidget *, const char *,
+			      struct newgame_args *);
+static void do_generate_sealed (GtkWidget *w, struct newgame_args *args);
 static void do_sideboard_cb (GtkWidget *w, struct newgame_args *args);
 static void newgame_dialog_ok (GtkWidget *w, struct newgame_args *args);
 static void got_connection_cb (struct newgame_args *, SOCKET,
@@ -69,13 +72,15 @@ void new_game_dialog (struct game *game)
     char *name, *text, *val, *ipaddr, buf[6];
     int i, port_num;
 
-    if (game->new_game_dialog != NULL)
+    if (game->new_game_dialog != NULL) {
+	gtk_window_present (GTK_WINDOW (game->new_game_dialog));
 	return;
-
+    }
     args = calloc (1, sizeof (struct newgame_args));
     args->game = game;
     dialog = gtk_window_new (GTK_WINDOW_TOPLEVEL);
     gtk_window_set_title (GTK_WINDOW (dialog), "New Game");
+    game_make_transient (game, dialog);
     gtk_object_set_user_data (GTK_OBJECT (dialog), args);
     vbox = gtk_vbox_new (FALSE, 6);
     gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
@@ -92,16 +97,20 @@ void new_game_dialog (struct game *game)
     gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
     gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 
-    hbox = gtk_hbox_new (FALSE, 30);
+    hbox = gtk_hbox_new (TRUE, 12);
     gtk_container_set_border_width (GTK_CONTAINER (hbox), 6);
     button = gtk_button_new_with_label ("Load...");
     gtk_signal_connect (GTK_OBJECT (button), "clicked",
 			GTK_SIGNAL_FUNC (do_load_deck), args);
-    gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 20);
+    gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 3);
+    button = new_wide_button ("Sealed Deck...", 6);
+    gtk_signal_connect (GTK_OBJECT (button), "clicked",
+			GTK_SIGNAL_FUNC (do_generate_sealed), args);
+    gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 3);
     button = gtk_button_new_with_label ("Sideboard...");
     gtk_signal_connect (GTK_OBJECT (button), "clicked",
 			GTK_SIGNAL_FUNC (do_sideboard_cb), args);
-    gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 20);
+    gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 3);
     gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 
     if (game->opponent == NULL) {
@@ -225,36 +234,82 @@ void new_game_dialog (struct game *game)
     gtk_widget_show_all (dialog);
 }
 
-static void do_load_deck (GtkWidget *w, struct newgame_args *args)
+#if (GTK_MAJOR_VERSION>2) || (GTK_MAJOR_VERSION==2&&GTK_MINOR_VERSION>2)
+static void
+load_deck_response (GtkDialog *dialog, gint arg, struct newgame_args *args)
 {
-    GtkWidget *filew;
-    GtkFileSelection *fsp;
-
-    filew = gtk_file_selection_new ("Load Deck");
-    gtk_object_set_user_data (GTK_OBJECT (filew), args);
-    fsp = GTK_FILE_SELECTION (filew);
-    gtk_signal_connect_object (GTK_OBJECT (fsp->ok_button), "clicked",
-			       GTK_SIGNAL_FUNC (file_ok_sel),
-			       GTK_OBJECT (filew));
-    gtk_signal_connect_object (GTK_OBJECT (fsp->cancel_button), "clicked",
-			       GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			       GTK_OBJECT (filew));
-    gtk_widget_show (filew);
+    const char *filename;
+    switch (arg) {
+    case GTK_RESPONSE_ACCEPT:
+	filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
+	finish_load_deck (GTK_WIDGET (dialog), filename, args);
+	break;
+    case GTK_RESPONSE_CANCEL:
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+	break;
+    }
 }
 
-static void file_ok_sel (GtkWidget *dialog, gpointer data)
+static void
+do_load_deck (GtkWidget *w, struct newgame_args *args)
+{
+    GtkWidget *dialog;
+    GtkFileFilter *filter;
+
+    dialog = gtk_file_chooser_dialog_new ("Load Deck",
+			GTK_WINDOW (args->game->new_game_dialog),
+			GTK_FILE_CHOOSER_ACTION_OPEN,
+			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+			GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+			NULL);
+    gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
+    filter = gtk_file_filter_new ();
+    gtk_file_filter_add_pattern (filter, "*.dec");
+    gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (dialog), filter);
+    gtk_signal_connect (GTK_OBJECT (dialog), "response",
+			GTK_SIGNAL_FUNC (load_deck_response), args);
+    gtk_widget_show_all (dialog);
+}
+#else
+static void
+load_deck_ok (GtkWidget *w, GtkWidget *dialog)
+{
+    const char *filename;
+    filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (dialog));
+    if (filename != NULL)
+	finish_load_deck (dialog, filename,
+			  gtk_object_get_user_data (GTK_OBJECT (dialog)));
+}
+
+static void
+do_load_deck (GtkWidget *w, struct newgame_args *args)
+{
+    GtkWidget *dialog;
+    GtkFileSelection *fsp;
+
+    dialog = gtk_file_selection_new ("Load Deck");
+    gtk_window_set_transient_for (GTK_WINDOW (dialog),
+				  GTK_WINDOW (args->game->new_game_dialog));
+    gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
+    gtk_object_set_user_data (GTK_OBJECT (dialog), args);
+    fsp = GTK_FILE_SELECTION (dialog);
+    gtk_signal_connect (GTK_OBJECT (fsp->ok_button), "clicked",
+			GTK_SIGNAL_FUNC (load_deck_ok), dialog);
+    gtk_signal_connect_object (GTK_OBJECT (fsp->cancel_button), "clicked",
+			       GTK_SIGNAL_FUNC (gtk_widget_destroy),
+			       GTK_OBJECT (dialog));
+    gtk_widget_show (dialog);
+}
+#endif
+
+static void finish_load_deck (GtkWidget *dialog, const char *filename,
+			      struct newgame_args *args)
 {
-    struct newgame_args *args;
     struct game *game;
-    const gchar *filename;
     struct deck *deck;
     struct player *me;
 
-    args = gtk_object_get_user_data (GTK_OBJECT (dialog));
     game = args->game;
-    filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (dialog));
-    if (filename == NULL)
-	return;
     deck = deck_read_file (filename, game->cardbase);
     if (deck == NULL) {
 	/* show an error */
@@ -268,11 +323,20 @@ static void file_ok_sel (GtkWidget *dialog, gpointer data)
 
     if (deck->name != NULL)
 	gtk_label_set_text (GTK_LABEL (args->deck_label), deck->name);
+
+    display_message (game, "%s has loaded a new deck for the next game.",
+		     me->name);
+    send_loaded_new_deck (game);
+}
+
+static void do_generate_sealed (GtkWidget *w, struct newgame_args *args)
+{
+    generate_sealed (args->game);
 }
 
 static void do_sideboard_cb (GtkWidget *w, struct newgame_args *args)
 {
-    do_sideboard_dialog (args->game);
+    do_sideboard_dialog (args->game, FALSE);
 }
 
 static void newgame_dialog_ok (GtkWidget *w, struct newgame_args *args)
@@ -287,7 +351,9 @@ static void newgame_dialog_ok (GtkWidget *w, struct newgame_args *args)
 	do_message_dialog ("Load a deck first.");
 	return;
     }
-    set_preference (game->prefs, deck_name_key, me->deck->filename);
+    /* Deck may be an (unsaved) sealed deck. */
+    if (me->deck->filename != NULL)
+	set_preference (game->prefs, deck_name_key, me->deck->filename);
     if (game->opponent != NULL) {
 	if ((game->opponent->flags & local_wants_new) != 0)
 	    return;
@@ -511,3 +577,22 @@ static int stop_waiting_for_call (struct newgame_args *args)
     gtk_label_set_text (GTK_LABEL (args->status_label), "");
     return (TRUE);
 }
+
+/*
+ * Display the name of the deck in the New Game dialog.
+ */
+void newgame_update_deck_label (struct game *game)
+{
+    struct newgame_args *args;
+    char *name;
+    struct player *me;
+
+    if (game->new_game_dialog == NULL)
+	return;
+    args = gtk_object_get_user_data (GTK_OBJECT (game->new_game_dialog));
+    name = "";
+    me = game->player[game->local_player];
+    if ((me->deck != NULL) && (me->deck->name != NULL))
+	name = me->deck->name;
+    gtk_label_set_text (GTK_LABEL (args->deck_label), name);
+}
