@@ -213,7 +213,7 @@ struct playarea *playarea_new (struct game *game)
 	gtk_signal_connect_object (GTK_OBJECT (mitem), "activate",
 				   GTK_SIGNAL_FUNC (change_color_callback),
 				   (gpointer) game);
-	gtk_object_set_user_data (GTK_OBJECT (mitem), (gpointer)(i+1));
+	gtk_object_set_user_data (GTK_OBJECT (mitem), GINT_TO_POINTER(i+1));
 	gtk_menu_append (GTK_MENU (color_menu), mitem);
 	gtk_widget_show (mitem);
     }
@@ -529,7 +529,9 @@ static void playarea_get_cardart_size (struct playarea *zone,
 {
     int width = rcard_width (zone) - 1 - 2*BORDER_WIDTH;
     *widthp = width;
-    *heightp = (int) ((double) width * 0.809);
+    *heightp = (int) ((double) width * 0.725);
+    /* Ensure enough space for three lines of text. */
+    if (*heightp < 33) *heightp = 33;
 }
 
 static int paint_card_picture (struct playarea *zone, struct cardinfo *info,
@@ -543,7 +545,7 @@ static int paint_card_picture (struct playarea *zone, struct cardinfo *info,
     playarea_get_cardart_size (zone, &pic_width, &pic_height);
     if (info->art == NULL)
 	info->art = cardart_load_cardart (info->name, info->expansion);
-    small_art = cardart_get_small_art (info->art, pic_width, pic_height);
+    small_art = cardart_get_small_art (info, pic_width, pic_height);
     if (small_art == NULL)
 	return (FALSE);
     gdk_pixbuf_render_to_drawable (small_art, zone->darea->window,
@@ -1143,7 +1145,7 @@ static void change_color_callback (struct game *game, GtkMenuItem *mitem)
 
     zone = game->playarea;
     tcard = playarea_get_table_card (zone, zone->drag_cid);
-    color = (int) gtk_object_get_user_data (GTK_OBJECT (mitem));
+    color = GPOINTER_TO_INT(gtk_object_get_user_data (GTK_OBJECT (mitem)));
     if (tcard->color != color) {
 	set_table_card_color (game, tcard, color, game->local_player);
 	send_sleight_card (game, tcard);
